@@ -1,6 +1,6 @@
 ############################################################
 #
-#   $Id: Simple.pm 642 2006-06-15 22:23:39Z nicolaw $
+#   $Id: Simple.pm 682 2006-06-24 22:58:27Z nicolaw $
 #   RRD::Simple - Simple interface to create and store data in RRD files
 #
 #   Copyright 2005,2006 Nicola Worthington
@@ -32,7 +32,7 @@ use File::Basename qw(fileparse dirname basename);
 use vars qw($VERSION $DEBUG $DEFAULT_DSTYPE
 			 @EXPORT @EXPORT_OK %EXPORT_TAGS @ISA);
 
-$VERSION = '1.38' || sprintf('%d', q$Revision: 642 $ =~ /(\d+)/g);
+$VERSION = '1.39' || sprintf('%d', q$Revision: 682 $ =~ /(\d+)/g);
 
 @ISA = qw(Exporter);
 @EXPORT = qw();
@@ -399,14 +399,18 @@ sub graph {
 	TRACE("graph() - \$cf = $cf");
 
 	# Create graphs which we have enough data to populate
-	my @rtn;
+	# Version 1.39 - Change the return from an array to a hash (semi backward compatible)
+#	my @rtn;
+	my %rtn;
 	for my $type (qw(day week month year 3years)) {
 		next if $period < _seconds_in($type);
 		TRACE("graph() - \$type = $type");
-		push @rtn, [ ($self->_create_graph($rrdfile, $type, $cf, @_)) ];
+#		push @rtn, [ ($self->_create_graph($rrdfile, $type, $cf, @_)) ];
+		$rtn{_alt_graph_name($type)} = [ ($self->_create_graph($rrdfile, $type, $cf, @_)) ];
 	}
 
-	return @rtn;
+#	return @rtn;
+	return %rtn;
 }
 
 
@@ -616,7 +620,8 @@ sub _create_graph {
 					"also an specified and valid array" if $^W;
 			} else {
 				for (my $i = 0; $i < @{$param{'source-labels'}}; $i++) {
-					$source_labels{$ds[$i]} = $param{'source-labels'}->[$i];
+					$source_labels{$ds[$i]} = $param{'source-labels'}->[$i]
+						if defined $ds[$i];
 				}
 			}
 		}
@@ -639,7 +644,8 @@ sub _create_graph {
 					"also an specified and valid array" if $^W;
 			} else {
 				for (my $i = 0; $i < @{$param{'source-drawtypes'}}; $i++) {
-					$source_drawtypes{$ds[$i]} = $param{'source-drawtypes'}->[$i];
+					$source_drawtypes{$ds[$i]} = $param{'source-drawtypes'}->[$i]
+						if defined $ds[$i];
 				}
 			}
 		}
@@ -731,19 +737,33 @@ sub _create_graph {
 				$stack
 			);
 
+		# New for version 1.39
+		# Return the min,max,last information in the graph() return @rtn
+		if ($RRDs::VERSION >= 1.2) {
+			push @cmd, sprintf('VDEF:%sMIN=%s,MINIMUM',$ds,$ds);
+			push @cmd, sprintf('VDEF:%sMAX=%s,MAXIMUM',$ds,$ds);
+			push @cmd, sprintf('VDEF:%sLAST=%s,LAST',$ds,$ds);
+			push @cmd, sprintf('PRINT:%sMIN:%s min %%1.2lf',$ds,$ds);
+			push @cmd, sprintf('PRINT:%sMAX:%s max %%1.2lf',$ds,$ds);
+			push @cmd, sprintf('PRINT:%sLAST:%s last %%1.2lf',$ds,$ds);
+		} else {
+			push @cmd, sprintf('PRINT:%s:MIN:%s min %%1.2lf',$ds,$ds);
+			push @cmd, sprintf('PRINT:%s:MAX:%s max %%1.2lf',$ds,$ds);
+			push @cmd, sprintf('PRINT:%s:LAST:%s last %%1.2lf',$ds,$ds);
+		}
+
 		# New for version 1.35
 		if ($extended_legend) {
 			if ($RRDs::VERSION >= 1.2) {
-				push @cmd, sprintf('VDEF:%sMIN=%s,MINIMUM',$ds,$ds);
-				push @cmd, sprintf('VDEF:%sMAX=%s,MAXIMUM',$ds,$ds);
-				push @cmd, sprintf('VDEF:%sLAST=%s,LAST',$ds,$ds);
+				# Moved the VDEFs to the block of code above which is
+				# always run, regardless of the extended legend
 				push @cmd, sprintf('GPRINT:%sMIN:   min\:%%10.2lf\g',$ds);
 				push @cmd, sprintf('GPRINT:%sMAX:   max\:%%10.2lf\g',$ds);
 				push @cmd, sprintf('GPRINT:%sLAST:   last\:%%10.2lf\l',$ds);
 			} else {
-				push @cmd, sprintf('GPRINT:%s:MIN:   min\:%%9.2lf\g',$ds);
-				push @cmd, sprintf('GPRINT:%s:MAX:   max\:%%9.2lf\g',$ds);
-				push @cmd, sprintf('GPRINT:%s:LAST:   last\:%%9.2lf\l',$ds);
+				push @cmd, sprintf('GPRINT:%s:MIN:   min\:%%10.2lf\g',$ds);
+				push @cmd, sprintf('GPRINT:%s:MAX:   max\:%%10.2lf\g',$ds);
+				push @cmd, sprintf('GPRINT:%s:LAST:   last\:%%10.2lf\l',$ds);
 			}
 		}
 	}
@@ -760,7 +780,7 @@ sub _create_graph {
 	my @rtn = RRDs::graph(@cmd);
 	my $error = RRDs::error;
 	croak($error) if $error;
-	return @rtn;
+	return ($image,@rtn);
 }
 
 
@@ -1165,12 +1185,13 @@ RRD::Simple - Simple interface to create and store data in RRD files
  # Generate graphs:
  # /var/tmp/myfile-daily.png, /var/tmp/myfile-weekly.png
  # /var/tmp/myfile-monthly.png, /var/tmp/myfile-annual.png
- my @rtn = $rrd->graph("myfile.rrd",
+ my %rtn = $rrd->graph("myfile.rrd",
              destination => "/var/tmp",
              title => "Network Interface eth0",
              vertical_label => "Bytes/Faults",
              interlaced => ""
          );
+ printf("Created %s\n",join(", ",map { $rtn{$_}->[0] } keys %rtn));
 
  # Return information about an RRD file
  my $info = $rrd->info("myfile.rrd");
@@ -1257,7 +1278,7 @@ C<$unixtime> is optional and will default to C<time()> (the current unixtime).
 Specifying this value will determine the date and time that your data point
 values will be stored against in the RRD file.
 
-If you try update a value for a data source that does not exist, it will
+If you try to update a value for a data source that does not exist, it will
 automatically be added for you. The data source type will be set to whatever
 is contained in the C<$RRD::Simple::DEFAULT_DSTYPE> variable. (See the
 VARIABLES section below).
@@ -1267,6 +1288,11 @@ are only updating pre-existing data source names using the C<sources> method.
 You can manually add new data sources to an RRD file by using the C<add_source>
 method, which requires you to explicitly set the data source type.
 
+If you try to update an RRD file that does not exist, it will attept to create
+the RRD file for you using the same behaviour as described above. A warning
+message will be displayed indicating that the RRD file is being created for
+you if have perl warnings turned on.
+
 =head2 last
 
  my $unixtime = $rrd->last($rrdfile);
@@ -1299,7 +1325,7 @@ add missing data sources.
 
 =head2 graph
 
- $rrd->graph($rrdfile,
+ my %rtn = $rrd->graph($rrdfile,
          destination => "/path/to/write/graph/images",
          basename => "graph_basename",
          sources => [ qw(source_name1 source_name2 source_name3) ],
@@ -1352,11 +1378,11 @@ plotted by default.
 
 =item source_colors
 
- $rrd->graph($rrdfile,
+ my %rtn = $rrd->graph($rrdfile,
          source_colors => [ qw(ff3333 ff00ff ffcc99) ],
      );
  
- $rrd->graph($rrdfile,
+ %rtn = $rrd->graph($rrdfile,
          source_colors => { source_name1 => "ff3333",
                             source_name2 => "ff00ff",
                             source_name3 => "ffcc99", },
@@ -1368,12 +1394,12 @@ lines. A selection of vivid primary colors will be set by default.
 
 =item source_labels
 
- $rrd->graph($rrdfile,
+ my %rtn = $rrd->graph($rrdfile,
          sources => [ qw(source_name1 source_name2 source_name3) ],
          source_labels => [ ("My Source 1","My Source Two","Source 3") ],
      );
  
- $rrd->graph($rrdfile,
+ %rtn = $rrd->graph($rrdfile,
          source_labels => { source_name1 => "My Source 1",
                             source_name2 => "My Source Two",
                             source_name3 => "Source 3", },
@@ -1390,17 +1416,17 @@ C<source_labels> parameter is specified.
 
 =item source_drawtypes
 
- $rrd->graph($rrdfile,
+ my %rtn = $rrd->graph($rrdfile,
          source_drawtypes => [ qw(LINE1 AREA LINE) ],
      );
  
- $rrd->graph($rrdfile,
+ %rtn = $rrd->graph($rrdfile,
          source_colors => { source_name1 => "LINE1",
                             source_name2 => "AREA",
                             source_name3 => "LINE", },
      );
  
- $rrd->graph($rrdfile,
+ %rtn = $rrd->graph($rrdfile,
          sources => [ qw(system user iowait idle) ]
          source_colors => [ qw(AREA STACK STACK STACK) ],
      );
@@ -1524,7 +1550,7 @@ L<http://www.rrdtool.org>, examples/*.pl
 
 =head1 VERSION
 
-$Id: Simple.pm 642 2006-06-15 22:23:39Z nicolaw $
+$Id: Simple.pm 682 2006-06-24 22:58:27Z nicolaw $
 
 =head1 AUTHOR
 
